@@ -11,7 +11,6 @@ from typing import Any, Optional
 
 from ..helpers import QNAME_PATTERN
 from .atomic_types import AtomicTypeMeta
-from .untyped import UntypedAtomic
 
 
 class AbstractQName(metaclass=AtomicTypeMeta):
@@ -61,7 +60,7 @@ class AbstractQName(metaclass=AtomicTypeMeta):
 
     @property
     def braced_uri_name(self) -> str:
-        return 'Q{%s}%s' % (self.uri, self.local_name) if self.uri else self.local_name
+        return 'Q{%s}%s' % (self.uri, self.local_name)
 
     def __repr__(self) -> str:
         return '%s(uri=%r, qname=%r)' % (self.__class__.__name__, self.uri, self.qname)
@@ -75,8 +74,6 @@ class AbstractQName(metaclass=AtomicTypeMeta):
     def __eq__(self, other: object) -> bool:
         if isinstance(other, AbstractQName):
             return self.uri == other.uri and self.local_name == other.local_name
-        elif isinstance(other, (str, UntypedAtomic)):
-            return other == self.qname
         raise TypeError("cannot compare {!r} to {!r}".format(type(self), type(other)))
 
 
