@@ -189,9 +189,11 @@ def format_digits(digits: str,
         else:
             separator = _separator.pop()
             chunks = fmt.split(separator)
-            repeat = len(chunks[-1])
-            if all(len(item) == repeat for item in chunks[1:-1]):
-                repeat += 1
+
+            if len(chunks[0]) > len(chunks[-1]):
+                repeat = None
+            elif all(len(item) == len(chunks[-1]) for item in chunks[1:-1]):
+                repeat = len(chunks[-1]) + 1
             else:
                 repeat = None
 
@@ -412,6 +414,9 @@ def parse_datetime_picture(picture: str) -> Tuple[List[str], List[str]]:
 
 
 def parse_datetime_marker(marker: str, dt: datetime.datetime, lang: Optional[str] = None) -> str:
+    min_width: int
+    max_width: Optional[int]
+
     component = marker[1]
     fmt_token = marker[2:-1]
 
@@ -453,9 +458,6 @@ def parse_datetime_marker(marker: str, dt: datetime.datetime, lang: Optional[str
     else:
         zero_cp, zero_ch = ord('0'), '0'
 
-    min_width: int
-    max_width: Optional[int]
-
     digits = sum(c.isdigit() for c in presentation)
     opt_digits = presentation.count('#')
     if not width or width == '*':
