@@ -157,7 +157,7 @@ class AttributeNode(XPathNode):
         return root_type.name == XSD_IDREF or root_type.name == XSD_IDREFS
 
     @property
-    def name(self) -> Optional[str]:
+    def name(self) -> str:
         return self._name
 
     @property
@@ -410,11 +410,17 @@ class ProcessingInstructionNode(XPathNode):
 
     @property
     def string_value(self) -> str:
-        return self.elem.text or ''
+        if hasattr(self.elem, 'target'):
+            return self.elem.text or ''
+
+        try:
+            return cast(str, self.elem.text).split(' ', maxsplit=1)[1]
+        except IndexError:
+            return ''
 
     @property
     def typed_value(self) -> str:
-        return self.elem.text or ''
+        return self.string_value
 
 
 class ElementNode(XPathNode):
@@ -704,8 +710,16 @@ class DocumentNode(XPathNode):
     @property
     def base_uri(self) -> Optional[str]:
         if not self.children:
+            # Fallback for not built documents
+            return self.document.getroot().get(XML_BASE)
+
+        for child in self.children:
+            if isinstance(child, ElementNode):
+                base_uri = child.elem.get(XML_BASE)
+                if base_uri is not None:
+                    return base_uri
+        else:
             return None
-        return self.getroot().base_uri
 
     def getroot(self) -> ElementNode:
         for child in self.children:
@@ -760,24 +774,50 @@ class DocumentNode(XPathNode):
 
     @property
     def string_value(self) -> str:
-        return ''.join(etree_iter_strings(self.document.getroot()))
+        if not self.children:
+            # Fallback for not built documents
+            root = self.document.getroot()
+            if root is None:
+                return ''
+            return ''.join(etree_iter_strings(root))
+        return ''.join(child.string_value for child in self.children)
 
     @property
     def typed_value(self) -> UntypedAtomic:
-        return UntypedAtomic(''.join(etree_iter_strings(self.document.getroot())))
+        return UntypedAtomic(self.string_value)
 
     @property
     def document_uri(self) -> Optional[str]:
+        base_uri = self.base_uri
+        if base_uri is None:
+            return None
+
         try:
-            uri = cast(str, self.document.getroot().attrib[XML_BASE])
-            parts = urlparse(uri)
-        except (KeyError, ValueError):
+            parts = urlparse(base_uri)
+        except ValueError:
             pass
         else:
             if parts.scheme and parts.netloc or parts.path.startswith('/'):
-                return uri
+                return base_uri
         return None
 
+    def is_extended(self) -> bool:
+        if self.document.getroot() is None:
+            return True
+        elif len(self.children) <= 1:
+            return False
+        elif not hasattr(self.document.getroot(), 'itersiblings'):
+            return True  # an xml.etree.ElementTree structure
+
+        root = None
+        for e in self.children:
+            if isinstance(e, ElementNode):
+                if root is not None:
+                    return True
+                root = e
+        else:
+            return False
+
 
 ###
 # Specialized element nodes
